	//preferences object
	const prefObj = new prefClass();

	//load function when program startup
	function loadingProgram()
	{
		var defaultEngine = "";

		//remove workaround "load" listener
		window.removeEventListener("load", loadingProgram, true);

		//set listener
		document.getElementById("txtSearchStr").onkeydown = txtInputSearchKey;

		//set enginelist		
		loadEngines(document.getElementById('engineList'));

		//get default engine
		defaultEngine = prefObj.getData("char", "browser.search.defaultengine");

		//get default search engine
		var defEngineObj = returnEngineObj(defaultEngine);

		if(defEngineObj)
		{
			defEngineObj.setAttribute("class","menuItemDefaultEngine");

			//set selected by default
			setSelectedEngine(defEngineObj);						
		}

		//get search history
		updateHistory(prefObj.getData("char", "extensions.easysearch.history"));

	}

	//setup search
	function loadEngines(engineList)
	{
		var myRdf = Components.classes["@mozilla.org/rdf/rdf-service;1"]
				   .getService(Components.interfaces.nsIRDFService);
		var engineDS = myRdf.GetDataSource("rdf:internetsearch");

		engineList.database.AddDataSource(engineDS);
		engineList.builder.rebuild();
	}

	//set selected engine
	function setSelectedEngine(inEngine)
	{
		if(inEngine)
		{
			document.getElementById("menuList").selectedItem = inEngine;
		}
	}

	//return specific engine object where engineObject.id = sEngine
	function returnEngineObj(sEngine)
	{
		var menuPopUp = document.getElementById('engineList');
        return menuPopUp.getElementsByAttribute("id", sEngine)[0];
	}

	//function for adding history
	function updateHistory(sHistory)
	{
		var historyArray = new Array();
		var historySearchArray = new Array(2);

		var historyPopUp = document.getElementById("browserHistory");

		//if there is a history
		if(sHistory)
		{
			historyArray = sHistory.split("||")
		
			clearAllChildNodes(historyPopUp);
			
			for(var i = 0; i < historyArray.length; i++)
			{
				//split url from searchstring
				historySearchArray = historyArray[i].split("|");

				//create menuListElement
				var menuListElement = document.createElement('menuitem');
				menuListElement.setAttribute("class","menuItemEngine");

				//get object from list
				var engineObj = returnEngineObj(historySearchArray[0]);

				if(engineObj) //if found object
				{
					menuListElement.setAttribute('src', engineObj.getAttribute("src"));
				}

				menuListElement.setAttribute('label', unescape(historySearchArray[1]));
				menuListElement.setAttribute('engine', historySearchArray[0]);
				menuListElement.setAttribute('oncommand', "selectPrevSearch(this)");

				historyPopUp.appendChild(menuListElement);
			}
		}
		else // if history is missing
		{
			//create menuListElement
			var menuListElement = document.createElement('menuitem');
			menuListElement.setAttribute('label', "No history available.");
			historyPopUp.appendChild(menuListElement);
		}
	}

	//add to search history
	function addToHistory(sUrl, sQuery)
	{
		var prefs = "";
		var sHistory = "";

		sHistory = prefObj.getData("char", "extensions.easysearch.history");

		if(sHistory)
		{
			sHistory = "||" + sHistory;
			sHistory = sUrl + "|" + sQuery + sHistory;
			sHistory = cutEndOfString(sHistory, "||", false);
		}
		else
		{
			sHistory = sUrl + "|" + sQuery;		
		}

		prefObj.setData("char", "extensions.easysearch.history", sHistory);
		updateHistory(sHistory);
	}

	//if selection from history
	function selectPrevSearch(inElement)
	{
		var txtSearch = document.getElementById("txtSearchStr");
		txtSearch.value = inElement.getAttribute("label");

		var defEngineObj = 
			returnEngineObj(inElement.getAttribute("engine"));

		//set selected engine
		setSelectedEngine(defEngineObj);
		txtSearch.focus();
	}

	//if event <enter>
	function txtInputSearchKey(event)
	{
		if (event.keyCode == 13) 
		{
			doSearch();
		}
	}

	//on search do search
	function doSearch()
	{
		var searchStr = "";

		//get search string
		searchStr = escape(document.getElementById("txtSearchStr").value);

		if(!searchStr)
		{
			return;			
		}

		var selectedItem 	= document.getElementById("menuList").selectedItem;
		var searchType		= selectedItem.getAttribute("searchtype");

		if(searchType == "page")
		{
			localSearch(searchStr);
			return;
		}
		else if(searchType == "internet")
		{
			internetSearch(selectedItem.getAttribute("id"), searchStr);
		}
	}

	//search the internet
	function internetSearch(sSearchUrl, sQuery)
	{
		var sUrl = "";
		if(!sSearchUrl)
		{
			return;
		}

		//search databasource
		var searchDS = Components.classes["@mozilla.org/rdf/datasource;1?name=internetsearch"]
				   	 .getService(Components.interfaces.nsIInternetSearchService);
		
		//if datasource exists
		if(searchDS)
		{
			try
			{
				sUrl = searchDS.GetInternetSearchURL(sSearchUrl, sQuery);
			}
			catch(e)
			{
				//failed
			}	

		}

		//if sidebar should be activated on search
		if(doOpenSideBar())
		{
			//if sidebar is opened, set focus on search and show search results
			var sidebarPanel = document.getElementById("urn:sidebar:panel:search");

			if(sidebarPanel)
			{
				SidebarSelectPanel(sidebarPanel, true, true);	
			}
		}

		addToHistory(sSearchUrl, sQuery);
        loadURI(sUrl);
	}
	
	//search local on my page
	function localSearch(sQuery)
	{
		var browserFinder 	= getBrowser().webBrowserFind;
		var boolMatchCase 	= document.getElementById("checkMatchCase").checked;	
		var boolWrap 		= document.getElementById("checkWrap").checked;	
		var boolBackwards 	= document.getElementById("checkBackwards").checked;	

		//save local search
		saveLocalSearch(sQuery, boolMatchCase, boolWrap, boolBackwards);

		try
		{
			//search in page
			findObj = browserFinder.QueryInterface(Components.interfaces.nsIWebBrowserFind);

			//set values
			findObj.searchString  = sQuery;
			findObj.matchCase     = boolMatchCase;
			findObj.wrapFind      = boolWrap;
			findObj.findBackwards = boolBackwards;
 			
 			//move next
 			var searchResult = findObj.findNext();
		
			if(!searchResult) // tell user
			{
				alert("The text you entered was not found.");
			}
		
		}
		catch(e)
		{
			alert(e);
		}
	}